function COOR=LDMI3(n,h)

%%  Author : W. Herreman
%
% n     : the degree of multipolar deformation (n>0). 
% h     : aspect ratio of cylindre
%
% COOR : a large structure containing all information of possibly 
%        resonant configurations, necessary fo post-processing


%% Step 1 : list of wavenumbers in scan
% -------------------------------------
%
% Given a cylinder with a certain aspect ratio, we find pairs of inertial waves
% that may exist within this cylinder, that may couple through LDMI. The azimuthal 
% wavenumbers vary in 
%
%   m1 = -n+1 to m1max
%   m2 = 1 to m1max+n 
%
% so that m1+n = m2 by convention. The vertical wavenumber can only take values 
% that are integer multiples (nz) of the fundamental wavelength 2 pi /h, and is the same for
% both modes
%
%   l1 = l2 = 2*pi/h* (1 to nzmax) 
%
% The radial structure of the waves is also quantified and we only allow
% for waves with radial labels
%
%   nr1 = nr2 = 1 to Nk 
%
% We fix m1max, nzmax, Nk in this program, to avoid having lengthy lists of
% variables when calling the function. We then create lists of wavenumbers
% for future use. 


Nk=2;       
nzmax = 10; 
m1max=10;   

m1list=(-n+1):m1max;     %list of wavenumbers m1
llist=(1:nzmax)*2*pi/h;  %list of wavenumbers l


% The radial wavenumbers are to be found as solutions of a dispersion
% relation. This relation is numerically solved, but initial estimates for 
% the first Nk radial wavenumber k1 and k2 are necessary. We obtain such estimates 
% by calculating the dispersion relation on a grid and find where it
% changes sign. Here we define this grid

lk=1000; 
klist=linspace(1,200,lk);  % for lk values of k between 1 and 200

% optimisation options

opt=optimset('TolFun',1e-6,'TolX',1e-6,'Display','off');


%% Step 2 : Solving dispersion relation & calculating matrix elements
% -------------------------------------------------------------------
%
% We find frequency wavenumber coordinates for each pairs of possibly resonant 
% waves and immediately calculate all the necessary matrix elements. In this way, 
% everything is calculated to make a libration frequency-scan in a postprocessing stage. 

tel=0;                 % counter
for tl=1:nzmax         % for all considered vertical wavelenghts
    l=llist(tl);       % vertical wavenumber
    
    for tm1=1:length(m1list) % for all considered azimuthal wavenumbers
        
        tic                 %follow evolution on screen
        tel=tel+1;
        [tel,nzmax*length(m1list)]
   
        m1=m1list(tm1);     %azimuthal wavenumbers
        m2=m1+n;
        
        
        % for each set of wavenumbers nr,m,nz we have two possible waves : 
        % 1 with positive frequency, and 1 with negative frequency.         
        % This means we can have 4 possible combinations, which explains 
        % 4 different classes
        
        for cl=1:4  % for all different classes
            
            
            %% A. Solve the dispersion relations : 
            %-------------------------------------
            % find the first Nk radial wavenumbers k and according frequencies lambda
            % for both modes
        
            switch cl
                case 1  %couplings of type lam1 < 0, lam2>0 (jomega>0)
                                       
                    [K1,LAM1]=solvedisper(1,-1);
                    [K2,LAM2]=solvedisper(2,+1);
                    
                case 2 %couplings of type lam1 > 0, lam2<0 (jomega<0)
                    
                    [K1,LAM1]=solvedisper(1,+1);
                    [K2,LAM2]=solvedisper(2,-1);
                    
                case 3 % couplings of type lam1 < 0, lam2>0 (jomega>0)
                                         
                    [K1,LAM1]=solvedisper(1,+1);
                    [K2,LAM2]=solvedisper(2,+1);
                    
                case 4 % couplings of type lam1 < 0, lam2>0 (jomega>0)
                     
                    [K1,LAM1]=solvedisper(1,-1);
                    [K2,LAM2]=solvedisper(2,-1);
            end
                    
            % solvedisper, is a nested function, defined beneath. 
            
            
            %% B. Matrix elements
            % --------------------
            % 
            % For each pair of modes, we calculate matrix elements that
            % allow to calculate growth rates in a post-processing step.
            %
            % We focus on central couplings, that have nr1 = nr2  and the 
            % largest growth rates. You can study non-central couplings by 
            % adding a supplementary loop (nr1 and nr2)               
            
            for nr=1:Nk  %for all central couplings 
             
               k1=K1(nr);
               k2=K2(nr);
               lam1=LAM1(nr);
               lam2=LAM2(nr);

               %coefficients in the radial wave profiles  
               cp1=2-lam1;
               cm1=2+lam1;
               cz1=-1i*k1*lam1/l;
               pp1=1i*k1*lam1^2/l^2;

               cp2=2-lam2;
               cm2=2+lam2;
               cz2=-1i*k2*lam2/l;
               pp2=1i*k2*lam2^2/l^2;

               %numerical integration for matrix elements (see down for nested functions)
               J11(nr)=quadl(@J11int,0,1);
               J22(nr)=quadl(@J22int,0,1);
               N12(nr)=quadl(@N12int,0,1);
               N21(nr)=quadl(@N21int,0,1);

               %boundary terms (see down for nested functions)
               BT12(nr)=BT12cal;
               BT21(nr)=BT21cal;

               %viscous coefficients (see down for nested function)
               dv=attvisc(m1,l,k1,lam1);
               Gz1(nr)=dv(1);             %surface damping on top and bottom (not used in article)
               Gr1(nr)=dv(2);             %surface damping on cylindrical sidewall 
               Gv1(nr)=dv(3);             %volume damping

               dv=attvisc(m2,l,k2,lam2);
               Gz2(nr)=dv(1);
               Gr2(nr)=dv(2);
               Gv2(nr)=dv(3);
               
            end
        
            % store usefull wavenumber frequency coordinates
            COOR(tl,tm1,cl).n=n;
            COOR(tl,tm1,cl).l=l;
            COOR(tl,tm1,cl).m1=m1;
            COOR(tl,tm1,cl).lam1=LAM1;
            COOR(tl,tm1,cl).lam2=LAM2;

            % store matrix elements        
            COOR(tl,tm1,cl).J11=J11;
            COOR(tl,tm1,cl).J22=J22;
            COOR(tl,tm1,cl).N12=N12;
            COOR(tl,tm1,cl).N21=N21;
            COOR(tl,tm1,cl).BT12=BT12;
            COOR(tl,tm1,cl).BT21=BT21;
            COOR(tl,tm1,cl).Gz1=Gz1;
            COOR(tl,tm1,cl).Gr1=Gr1;
            COOR(tl,tm1,cl).Gv1=Gv1;
            COOR(tl,tm1,cl).Gz2=Gz2;
            COOR(tl,tm1,cl).Gr2=Gr2;
            COOR(tl,tm1,cl).Gv2=Gv2;

        end
        
       toc 
    end
end

%% -------------Nested functions used above--------------
    
    %% Solving dispersion relation 
    function [KK,LLAM]=solvedisper(wave,fs)
            
        %chose wave
        switch wave
            case 1
                m=m1;
            case 2
                m=m2;
        end
            
        %find initial estimates for radial wavenumber
        res=disp(klist);                                   % evaluate dispersion relation on 1D grid
        in=find(sign(res(1:(lk-1)))~=sign(res(2:(lk))));   % find where the sign change = close to a zero
        KK=klist(in(1:Nk));                                % store the first Nk values of k where the sign changes
        
        %optimisation using fzero
        for nrr=1:Nk
            KK(nrr)=fzero(@disp,KK(nrr),opt);           
        end
        LLAM=fs*sqrt(4./(l^2+KK.^2))*l; %according frequency, with the chosen sign.
            
        %nested-nested function to evaluate dispersion relation
        function res=disp(k)
            lam=fs*sqrt(4./(l^2+k.^2))*l;
            res=(2-lam).*besselj(m+1,k)+(2+lam).*besselj(m-1,k);
        end
            
    end
   
       
    %% Radial profile functions
    function f=ur1(r)
        f=1/2*(cp1*besselj(m1+1,k1*r)+cm1*besselj(m1-1,k1*r));
    end
    function f=ur2(r)
        f=1/2*(cp2*besselj(m2+1,k2*r)+cm2*besselj(m2-1,k2*r));
    end
    function f=uth1(r)
        f=-1i/2*(cp1*besselj(m1+1,k1*r)-cm1*besselj(m1-1,k1*r));
    end
    function f=uth2(r)
        f=-1i/2*(cp2*besselj(m2+1,k2*r)-cm2*besselj(m2-1,k2*r));
    end
    function f=uz1(r)
        f=cz1*besselj(m1,k1*r);
    end
    function f=uz2(r)
        f=cz2*besselj(m2,k2*r);
    end
    function f=p1(r)
        f=pp1*besselj(m1,k1*r);
    end
    function f=p2(r)
        f=pp2*besselj(m2,k2*r);
    end
    function f=drur1(r)
        f=k1/4*(-cp1*besselj(m1+2,k1*r)+(cp1-cm1)*besselj(m1,k1*r)+cm1*besselj(m1-2,k1*r));
    end
    function f=drur2(r)
        f=k2/4*(-cp2*besselj(m2+2,k2*r)+(cp2-cm2)*besselj(m2,k2*r)+cm2*besselj(m2-2,k2*r));
    end
    function f=druth1(r)
        f=-1i*k1/4*(-cp1*besselj(m1+2,k1*r)+(cp1+cm1)*besselj(m1,k1*r)-cm1*besselj(m1-2,k1*r));
    end
    function f=druth2(r)
        f=-1i*k2/4*(-cp2*besselj(m2+2,k2*r)+(cp2+cm2)*besselj(m2,k2*r)-cm2*besselj(m2-2,k2*r));
    end
    function f=druz1(r)
        f=k1/2*cz1*(besselj(m1-1,k1*r)-besselj(m1+1,k1*r));
    end
    function f=druz2(r)
        f=k2/2*cz2*(besselj(m2-1,k2*r)-besselj(m2+1,k2*r));
    end

    %% Integrandum functions for matrix elements J11, J22, N12 and N21
    function f=J11int(r)
        f=( (conj(ur1(r)).*ur1(r))+(conj(uth1(r)).*uth1(r))+(conj(uz1(r)).*uz1(r)) ).*r;
    end
    function f=J22int(r)
        f=( (conj(ur2(r)).*ur2(r))+(conj(uth2(r)).*uth2(r))+(conj(uz2(r)).*uz2(r)) ).*r;
    end
    function f=N12int(r)
        f=  (-1i)*(conj(ur1(r) ).*(-(r.^(n-1)).*drur2(r) -( m2+n-1)*(r.^(n-2)).*ur2(r)  +1i*(n-2)*(r.^(n-2)).*uth2(r)  )  ).*r;
        f=f+(-1i)*(conj(uth1(r)).*(-(r.^(n-1)).*druth2(r)+(-m2+n-1)*(r.^(n-2)).*uth2(r) +1i*(n  )*(r.^(n-2)).*ur2(r)        ) ).*r;
        f=f+(-1i)*(conj(uz1(r) ).*(-(r.^(n-1)).*druz2(r) +(-m2    )*(r.^(n-2)).*uz2(r) ) ).*r;
    end
    function f=N21int(r)
        f=  (1i)*(conj(ur2(r) ).*(-(r.^(n-1)).*drur1(r) +( m1-n+1)*(r.^(n-2)).*ur1(r)  -1i*(n-2)*(r.^(n-2)).*uth1(r)  )  ).*r;
        f=f+(1i)*(conj(uth2(r)).*(-(r.^(n-1)).*druth1(r)+( m1+n-1)*(r.^(n-2)).*uth1(r) -1i*(n  )*(r.^(n-2)).*ur1(r)        ) ).*r;
        f=f+(1i)*(conj(uz2(r) ).*(-(r.^(n-1)).*druz1(r) +( m1    )*(r.^(n-2)).*uz1(r) ) ).*r;
    end
   
   %% Boundary terms
    function f=BT12cal
        f=conj(p1(1))*(-1/n*drur2(1)-1i*uth2(1));
    end
    function f=BT21cal
        f=conj(p2(1))*(-1/n*drur1(1)+1i*uth1(1));
    end

    %% Viscous attenuation
    function dv=attvisc(m,l,k,la)

        opt='k';
        dv=zeros(1,3);
        switch opt
    
            case 'k' %kudlich estimate

                if sign(la)>0

                    dv(1)=(4-la^2)/(4*sqrt(2)*(m^2+l^2-m*la/2))*((1+1i)*(2-la)/sqrt(2+la)*(m^2+l^2-2*m*la/(2-la)) + (1-1i)*(2+la)/sqrt(2-la)*(m^2+l^2-2*m*la/(2+la)));
                    dv(2)=(4-la^2)/(4*sqrt(2)*(m^2+l^2-m*la/2))*(1+1i)*sqrt(la)*(m^2+l^2);   
                else
                    dv(1)=conj((4-la^2)/(4*sqrt(2)*(m^2+l^2-m*la/2))*((1+1i)*(2+la)/sqrt(2-la)*(m^2+l^2-2*m*la/(2+la)) + (1-1i)*(2-la)/sqrt(2+la)*(m^2+l^2-2*m*la/(2-la))));
                    dv(2)=conj((4-la^2)/(4*sqrt(2)*(m^2+l^2-m*la/2))*(1+1i)*sqrt(abs(la))*(m^2+l^2));   
                end    

            case 'w' %wedemeyer estimate
   
                if sign(la)>0

                    dv(1)=     (4-la^2)/(4*sqrt(2)*(m^2+l^2-m*la/2))*((1+1i)*(2-la)/sqrt(2+la)*(m^2+l^2) + (1-1i)*(2+la)/sqrt(2-la)*(m^2+l^2));
                    dv(2)=     (4-la^2)/(4*sqrt(2)*(m^2+l^2-m*la/2))*(1+1i)*sqrt(la)*(m^2+l^2);   
                else
                    dv(1)=conj((4-la^2)/(4*sqrt(2)*(m^2+l^2-m*la/2))*((1+1i)*(2+la)/sqrt(2-la)*(m^2+l^2) + (1-1i)*(2-la)/sqrt(2+la)*(m^2+l^2)));
                    dv(2)=conj((4-la^2)/(4*sqrt(2)*(m^2+l^2-m*la/2))*(1+1i)*sqrt(abs(la))*(m^2+l^2));   
                end    
        end
        dv(3)=(k^2+l^2);
    end

end


